@@ -44,10 +44,10 @@ import org.apache.commons.compress.archivers.ArchiveEntry;
  * 
  * Due to the limitation of the file name length to 16 bytes GNU and
  * BSD has their own variants of this format. Currently Commons
- * Compress can read but not write the GNU variant and doesn't support
- * the BSD variant at all.
+ * Compress can read but not write the GNU variant.  It fully supports
+ * the BSD variant.
  * 
- * @see http://www.freebsd.org/cgi/man.cgi?query=ar&sektion=5
+ * @see <a href="http://www.freebsd.org/cgi/man.cgi?query=ar&sektion=5">ar man page</a>
  *
  * @Immutable
  */
@@ -115,12 +115,10 @@ public class ArArchiveEntry implements ArchiveEntry {
              0, 0, DEFAULT_MODE, inputFile.lastModified() / 1000);
     }
 
-    /** {@inheritDoc} */
     public long getSize() {
         return this.getLength();
     }
 
-    /** {@inheritDoc} */
     public String getName() {
         return name;
     }
@@ -144,7 +142,6 @@ public class ArArchiveEntry implements ArchiveEntry {
         return lastModified;
     }
 
-    /** {@inheritDoc} */
     public Date getLastModifiedDate() {
         return new Date(1000 * getLastModified());
     }
@@ -153,12 +150,11 @@ public class ArArchiveEntry implements ArchiveEntry {
         return length;
     }
 
-    /** {@inheritDoc} */
     public boolean isDirectory() {
         return false;
     }
 
-    /** {@inheritDoc} */
+    @Override
     public int hashCode() {
         final int prime = 31;
         int result = 1;
@@ -166,7 +162,7 @@ public class ArArchiveEntry implements ArchiveEntry {
         return result;
     }
 
-    /** {@inheritDoc} */
+    @Override
     public boolean equals(Object obj) {
         if (this == obj) {
             return true;
