@@ -30,7 +30,7 @@ import org.apache.commons.compress.archivers.ArchiveEntry;
  * 
  * <ul>
  * <li>"070701" ASCII for new portable format</li>
- * <li>"070702" ASCII for new portable format with CRC format</li>
+ * <li>"070702" ASCII for new portable format with CRC</li>
  * <li>"070707" ASCII for old ascii (also known as Portable ASCII, odc or old
  * character format</li>
  * <li>070707 binary for old binary</li>
@@ -577,7 +577,6 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
         return this.mtime;
     }
 
-    /** {@inheritDoc} */
     public Date getLastModifiedDate() {
         return new Date(1000 * getTime());
     }
@@ -597,7 +596,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a block device.
      */
     public boolean isBlockDevice() {
-        return (this.mode & S_IFMT) == C_ISBLK;
+        return CpioUtil.fileType(mode) == C_ISBLK;
     }
 
     /**
@@ -606,7 +605,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a character device.
      */
     public boolean isCharacterDevice() {
-        return (this.mode & S_IFMT) == C_ISCHR;
+        return CpioUtil.fileType(mode) == C_ISCHR;
     }
 
     /**
@@ -615,7 +614,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a directory.
      */
     public boolean isDirectory() {
-        return (this.mode & S_IFMT) == C_ISDIR;
+        return CpioUtil.fileType(mode) == C_ISDIR;
     }
 
     /**
@@ -624,7 +623,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a network device.
      */
     public boolean isNetwork() {
-        return (this.mode & S_IFMT) == C_ISNWK;
+        return CpioUtil.fileType(mode) == C_ISNWK;
     }
 
     /**
@@ -633,7 +632,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a pipe.
      */
     public boolean isPipe() {
-        return (this.mode & S_IFMT) == C_ISFIFO;
+        return CpioUtil.fileType(mode) == C_ISFIFO;
     }
 
     /**
@@ -642,7 +641,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a regular file.
      */
     public boolean isRegularFile() {
-        return (this.mode & S_IFMT) == C_ISREG;
+        return CpioUtil.fileType(mode) == C_ISREG;
     }
 
     /**
@@ -651,7 +650,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a socket.
      */
     public boolean isSocket() {
-        return (this.mode & S_IFMT) == C_ISSOCK;
+        return CpioUtil.fileType(mode) == C_ISSOCK;
     }
 
     /**
@@ -660,7 +659,7 @@ public class CpioArchiveEntry implements CpioConstants, ArchiveEntry {
      * @return TRUE if this entry is a symbolic link.
      */
     public boolean isSymbolicLink() {
-        return (this.mode & S_IFMT) == C_ISLNK;
+        return CpioUtil.fileType(mode) == C_ISLNK;
     }
 
     /**
