@@ -115,45 +115,48 @@ import org.apache.commons.compress.utils.ArchiveUtils;
 
 public class TarArchiveEntry implements TarConstants, ArchiveEntry {
     /** The entry's name. */
-    private String name;
+    private String name = "";
 
     /** The entry's permission mode. */
     private int mode;
 
     /** The entry's user id. */
-    private int userId;
+    private int userId = 0;
 
     /** The entry's group id. */
-    private int groupId;
+    private int groupId = 0;
 
     /** The entry's size. */
-    private long size;
+    private long size = 0;
 
     /** The entry's modification time. */
     private long modTime;
 
+    /** If the header checksum is reasonably correct. */
+    private boolean checkSumOK;
+
     /** The entry's link flag. */
     private byte linkFlag;
 
     /** The entry's link name. */
-    private String linkName;
+    private String linkName = "";
 
     /** The entry's magic tag. */
-    private String magic;
+    private String magic = MAGIC_POSIX;
     /** The version of the format */
-    private String version;
+    private String version = VERSION_POSIX;
 
     /** The entry's user name. */
     private String userName;
 
     /** The entry's group name. */
-    private String groupName;
+    private String groupName = "";
 
     /** The entry's major device number. */
-    private int devMajor;
+    private int devMajor = 0;
 
     /** The entry's minor device number. */
-    private int devMinor;
+    private int devMinor = 0;
 
     /** If an extension sparse header follows. */
     private boolean isExtended;
@@ -162,7 +165,7 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
     private long realSize;
 
     /** The entry's file reference */
-    private File file;
+    private final File file;
 
     /** Maximum length of a user's name in the tar file */
     public static final int MAX_NAMELEN = 31;
@@ -180,21 +183,13 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
      * Construct an empty entry and prepares the header values.
      */
     private TarArchiveEntry() {
-        this.magic = MAGIC_POSIX;
-        this.version = VERSION_POSIX;
-        this.name = "";
-        this.linkName = "";
-
         String user = System.getProperty("user.name", "");
 
         if (user.length() > MAX_NAMELEN) {
             user = user.substring(0, MAX_NAMELEN);
         }
 
-        this.userId = 0;
-        this.groupId = 0;
         this.userName = user;
-        this.groupName = "";
         this.file = null;
     }
 
@@ -224,18 +219,11 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
         name = normalizeFileName(name, preserveLeadingSlashes);
         boolean isDir = name.endsWith("/");
 
-        this.devMajor = 0;
-        this.devMinor = 0;
         this.name = name;
         this.mode = isDir ? DEFAULT_DIR_MODE : DEFAULT_FILE_MODE;
         this.linkFlag = isDir ? LF_DIR : LF_NORMAL;
-        this.userId = 0;
-        this.groupId = 0;
-        this.size = 0;
         this.modTime = (new Date()).getTime() / MILLIS_PER_SECOND;
-        this.linkName = "";
         this.userName = "";
-        this.groupName = "";
     }
 
     /**
@@ -245,7 +233,21 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
      * @param linkFlag the entry link flag.
      */
     public TarArchiveEntry(String name, byte linkFlag) {
-        this(name);
+        this(name, linkFlag, false);
+    }
+
+    /**
+     * Construct an entry with a name and a link flag.
+     *
+     * @param name the entry name
+     * @param linkFlag the entry link flag.
+     * @param preserveLeadingSlashes whether to allow leading slashes
+     * in the name.
+     * 
+     * @since 1.5
+     */
+    public TarArchiveEntry(String name, byte linkFlag, boolean preserveLeadingSlashes) {
+        this(name, preserveLeadingSlashes);
         this.linkFlag = linkFlag;
         if (linkFlag == LF_GNUTYPE_LONGNAME) {
             magic = MAGIC_GNU;
@@ -272,12 +274,8 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
      * @param fileName the name to be used for the entry.
      */
     public TarArchiveEntry(File file, String fileName) {
-        this();
-
         this.file = file;
 
-        this.linkName = "";
-
         if (file.isDirectory()) {
             this.mode = DEFAULT_DIR_MODE;
             this.linkFlag = LF_DIR;
@@ -288,7 +286,6 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
             } else {
                 this.name = fileName;
             }
-            this.size = 0;
         } else {
             this.mode = DEFAULT_FILE_MODE;
             this.linkFlag = LF_NORMAL;
@@ -297,8 +294,7 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
         }
 
         this.modTime = file.lastModified() / MILLIS_PER_SECOND;
-        this.devMajor = 0;
-        this.devMinor = 0;
+        this.userName = "";
     }
 
     /**
@@ -319,7 +315,7 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
      *
      * @param headerBuf The header bytes from a tar archive entry.
      * @param encoding encoding to use for file names
-     * @since Commons Compress 1.4
+     * @since 1.4
      * @throws IllegalArgumentException if any of the numeric fields have an invalid format
      */
     public TarArchiveEntry(byte[] headerBuf, ZipEncoding encoding)
@@ -545,11 +541,21 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
         return new Date(modTime * MILLIS_PER_SECOND);
     }
 
-    /** {@inheritDoc} */
     public Date getLastModifiedDate() {
         return getModTime();
     }
 
+    /**
+     * Get this entry's checksum status.
+     *
+     * @return if the header checksum is reasonably correct
+     * @see TarUtils#verifyCheckSum(byte[])
+     * @since 1.5
+     */
+    public boolean isCheckSumOK() {
+        return checkSumOK;
+    }
+
     /**
      * Get this entry's file.
      *
@@ -668,6 +674,16 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
         return linkFlag == LF_GNUTYPE_SPARSE;
     }
 
+    /**
+     * Indicate if this entry is a GNU long linkname block
+     *
+     * @return true if this is a long name extension provided by GNU tar
+     */
+    public boolean isGNULongLinkEntry() {
+        return linkFlag == LF_GNUTYPE_LONGLINK
+            && name.equals(GNU_LONGLINK);
+    }
+
     /**
      * Indicate if this entry is a GNU long name block
      *
@@ -675,7 +691,7 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
      */
     public boolean isGNULongNameEntry() {
         return linkFlag == LF_GNUTYPE_LONGNAME
-            && name.toString().equals(GNU_LONGLINK);
+            && name.equals(GNU_LONGLINK);
     }
 
     /**
@@ -915,7 +931,7 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
      *
      * @param header The tar entry header buffer to get information from.
      * @param encoding encoding to use for file names
-     * @since Commons Compress 1.4
+     * @since 1.4
      * @throws IllegalArgumentException if any of the numeric fields
      * have an invalid format
      */
@@ -942,6 +958,7 @@ public class TarArchiveEntry implements TarConstants, ArchiveEntry {
         offset += SIZELEN;
         modTime = TarUtils.parseOctalOrBinary(header, offset, MODTIMELEN);
         offset += MODTIMELEN;
+        checkSumOK = TarUtils.verifyCheckSum(header);
         offset += CHKSUMLEN;
         linkFlag = header[offset++];
         linkName = oldStyle ? TarUtils.parseName(header, offset, NAMELEN)
