@@ -17,6 +17,8 @@
  */
 package org.apache.commons.compress.archivers.zip;
 
+import org.apache.commons.compress.archivers.ArchiveEntry;
+
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -24,7 +26,6 @@ import java.util.Date;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.zip.ZipException;
-import org.apache.commons.compress.archivers.ArchiveEntry;
 
 /**
  * Extension that adds better handling of extra fields and provides
@@ -49,12 +50,13 @@ import org.apache.commons.compress.archivers.ArchiveEntry;
  * @NotThreadSafe
  */
 public class ZipArchiveEntry extends java.util.zip.ZipEntry
-    implements ArchiveEntry, Cloneable {
+    implements ArchiveEntry {
 
     public static final int PLATFORM_UNIX = 3;
     public static final int PLATFORM_FAT  = 0;
     private static final int SHORT_MASK = 0xFFFF;
     private static final int SHORT_SHIFT = 16;
+    private static final byte[] EMPTY = new byte[0];
 
     /**
      * The {@link java.util.zip.ZipEntry} base class only supports
@@ -68,10 +70,18 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      */
     private int method = -1;
 
+    /**
+     * The {@link java.util.zip.ZipEntry#setSize} method in the base
+     * class throws an IllegalArgumentException if the size is bigger
+     * than 2GB for Java versions < 7.  Need to keep our own size
+     * information for Zip64 support.
+     */
+    private long size = SIZE_UNKNOWN;
+
     private int internalAttributes = 0;
     private int platform = PLATFORM_FAT;
     private long externalAttributes = 0;
-    private LinkedHashMap/*<ZipShort, ZipExtraField>*/ extraFields = null;
+    private LinkedHashMap<ZipShort, ZipExtraField> extraFields = null;
     private UnparseableExtraFieldData unparseableExtra = null;
     private String name = null;
     private byte[] rawName = null;
@@ -112,6 +122,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
             setExtra();
         }
         setMethod(entry.getMethod());
+        this.size = entry.getSize();
     }
 
     /**
@@ -159,6 +170,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * Overwrite clone.
      * @return a cloned copy of this ZipArchiveEntry
      */
+    @Override
     public Object clone() {
         ZipArchiveEntry e = (ZipArchiveEntry) super.clone();
 
@@ -174,8 +186,9 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      *
      * @return compression method
      *
-     * @since Apache Commons Compress 1.1
+     * @since 1.1
      */
+    @Override
     public int getMethod() {
         return method;
     }
@@ -185,8 +198,9 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      *
      * @param method compression method
      *
-     * @since Apache Commons Compress 1.1
+     * @since 1.1
      */
+    @Override
     public void setMethod(int method) {
         if (method < 0) {
             throw new IllegalArgumentException(
@@ -253,12 +267,24 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
             (int) ((getExternalAttributes() >> SHORT_SHIFT) & SHORT_MASK);
     }
 
+    /**
+     * Returns true if this entry represents a unix symlink,
+     * in which case the entry's content contains the target path
+     * for the symlink.
+     *
+     * @since 1.5
+     * @return true if the entry represents a unix symlink, false otherwise.
+     */
+    public boolean isUnixSymlink() {
+        return (getUnixMode() & UnixStat.LINK_FLAG) == UnixStat.LINK_FLAG;
+    }
+
     /**
      * Platform specification to put into the &quot;version made
      * by&quot; part of the central file header.
      *
      * @return PLATFORM_FAT unless {@link #setUnixMode setUnixMode}
-     * has been called, in which case PLATORM_UNIX will be returned.
+     * has been called, in which case PLATFORM_UNIX will be returned.
      */
     public int getPlatform() {
         return platform;
@@ -277,12 +303,12 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * @param fields an array of extra fields
      */
     public void setExtraFields(ZipExtraField[] fields) {
-        extraFields = new LinkedHashMap();
-        for (int i = 0; i < fields.length; i++) {
-            if (fields[i] instanceof UnparseableExtraFieldData) {
-                unparseableExtra = (UnparseableExtraFieldData) fields[i];
+        extraFields = new LinkedHashMap<ZipShort, ZipExtraField>();
+        for (ZipExtraField field : fields) {
+            if (field instanceof UnparseableExtraFieldData) {
+                unparseableExtra = (UnparseableExtraFieldData) field;
             } else {
-                extraFields.put(fields[i].getHeaderId(), fields[i]);
+                extraFields.put(field.getHeaderId(), field);
             }
         }
         setExtra();
@@ -303,7 +329,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * exists.
      * @return an array of the extra fields
      *
-     * @since Apache Commons Compress 1.1
+     * @since 1.1
      */
     public ZipExtraField[] getExtraFields(boolean includeUnparseable) {
         if (extraFields == null) {
@@ -311,11 +337,12 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
                 ? new ZipExtraField[0]
                 : new ZipExtraField[] { unparseableExtra };
         }
-        List result = new ArrayList(extraFields.values());
+        List<ZipExtraField> result =
+            new ArrayList<ZipExtraField>(extraFields.values());
         if (includeUnparseable && unparseableExtra != null) {
             result.add(unparseableExtra);
         }
-        return (ZipExtraField[]) result.toArray(new ZipExtraField[0]);
+        return result.toArray(new ZipExtraField[0]);
     }
 
     /**
@@ -331,7 +358,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
             unparseableExtra = (UnparseableExtraFieldData) ze;
         } else {
             if (extraFields == null) {
-                extraFields = new LinkedHashMap();
+                extraFields = new LinkedHashMap<ZipShort, ZipExtraField>();
             }
             extraFields.put(ze.getHeaderId(), ze);
         }
@@ -349,8 +376,8 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
         if (ze instanceof UnparseableExtraFieldData) {
             unparseableExtra = (UnparseableExtraFieldData) ze;
         } else {
-            LinkedHashMap copy = extraFields;
-            extraFields = new LinkedHashMap();
+            LinkedHashMap<ZipShort, ZipExtraField> copy = extraFields;
+            extraFields = new LinkedHashMap<ZipShort, ZipExtraField>();
             extraFields.put(ze.getHeaderId(), ze);
             if (copy != null) {
                 copy.remove(ze.getHeaderId());
@@ -377,7 +404,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
     /**
      * Removes unparseable extra field data.
      *
-     * @since Apache Commons Compress 1.1
+     * @since 1.1
      */
     public void removeUnparseableExtraFieldData() {
         if (unparseableExtra == null) {
@@ -394,7 +421,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      */
     public ZipExtraField getExtraField(ZipShort type) {
         if (extraFields != null) {
-            return (ZipExtraField) extraFields.get(type);
+            return extraFields.get(type);
         }
         return null;
     }
@@ -404,7 +431,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      *
      * @return null if no such field exists.
      *
-     * @since Apache Commons Compress 1.1
+     * @since 1.1
      */
     public UnparseableExtraFieldData getUnparseableExtraFieldData() {
         return unparseableExtra;
@@ -418,6 +445,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * @throws RuntimeException if the bytes cannot be parsed
      * @throws RuntimeException on error
      */
+    @Override
     public void setExtra(byte[] extra) throws RuntimeException {
         try {
             ZipExtraField[] local =
@@ -425,7 +453,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
                                       ExtraFieldUtils.UnparseableExtraField.READ);
             mergeExtraFields(local, true);
         } catch (ZipException e) {
-            // actually this is not be possible as of Commons Compress 1.1
+            // actually this is not possible as of Commons Compress 1.1
             throw new RuntimeException("Error parsing extra fields for entry: "
                                        + getName() + " - " + e.getMessage(), e);
         }
@@ -461,7 +489,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      */
     public byte[] getLocalFileDataExtra() {
         byte[] extra = getExtra();
-        return extra != null ? extra : new byte[0];
+        return extra != null ? extra : EMPTY;
     }
 
     /**
@@ -476,6 +504,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * Get the name of the entry.
      * @return the entry name
      */
+    @Override
     public String getName() {
         return name == null ? super.getName() : name;
     }
@@ -484,6 +513,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * Is this entry a directory?
      * @return true if the entry is a directory
      */
+    @Override
     public boolean isDirectory() {
         return getName().endsWith("/");
     }
@@ -493,9 +523,36 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * @param name the name to use
      */
     protected void setName(String name) {
+        if (name != null && getPlatform() == PLATFORM_FAT
+            && name.indexOf("/") == -1) {
+            name = name.replace('\\', '/');
+        }
         this.name = name;
     }
 
+    /**
+     * Gets the uncompressed size of the entry data.
+     * @return the entry size
+     */
+    @Override
+    public long getSize() {
+        return size;
+    }
+
+    /**
+     * Sets the uncompressed size of the entry data.
+     * @param size the uncompressed size in bytes
+     * @exception IllegalArgumentException if the specified size is less
+     *            than 0
+     */
+    @Override
+    public void setSize(long size) {
+        if (size < 0) {
+            throw new IllegalArgumentException("invalid entry size");
+        }
+        this.size = size;
+    }
+
     /**
      * Sets the name using the raw bytes and the string created from
      * it by guessing or using the configured encoding.
@@ -503,7 +560,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * the guessed or configured encoding
      * @param rawName the bytes originally read as name from the
      * archive
-     * @since Apache Commons Compress 1.2
+     * @since 1.2
      */
     protected void setName(String name, byte[] rawName) {
         setName(name);
@@ -517,7 +574,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * <p>This method will return null if this instance has not been
      * read from an archive.</p>
      *
-     * @since Apache Commons Compress 1.2
+     * @since 1.2
      */
     public byte[] getRawName() {
         if (rawName != null) {
@@ -533,6 +590,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
      * This uses the name as the hashcode.
      * @return a hashcode.
      */
+    @Override
     public int hashCode() {
         // this method has severe consequences on performance. We cannot rely
         // on the super.hashCode() method since super.getName() always return
@@ -543,7 +601,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
 
     /**
      * The "general purpose bit" field.
-     * @since Apache Commons Compress 1.1
+     * @since 1.1
      */
     public GeneralPurposeBit getGeneralPurposeBit() {
         return gpb;
@@ -551,7 +609,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
 
     /**
      * The "general purpose bit" field.
-     * @since Apache Commons Compress 1.1
+     * @since 1.1
      */
     public void setGeneralPurposeBit(GeneralPurposeBit b) {
         gpb = b;
@@ -570,21 +628,21 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
         if (extraFields == null) {
             setExtraFields(f);
         } else {
-            for (int i = 0; i < f.length; i++) {
+            for (ZipExtraField element : f) {
                 ZipExtraField existing;
-                if (f[i] instanceof UnparseableExtraFieldData) {
+                if (element instanceof UnparseableExtraFieldData) {
                     existing = unparseableExtra;
                 } else {
-                    existing = getExtraField(f[i].getHeaderId());
+                    existing = getExtraField(element.getHeaderId());
                 }
                 if (existing == null) {
-                    addExtraField(f[i]);
+                    addExtraField(element);
                 } else {
                     if (local) {
-                        byte[] b = f[i].getLocalFileDataData();
+                        byte[] b = element.getLocalFileDataData();
                         existing.parseFromLocalFileData(b, 0, b.length);
                     } else {
-                        byte[] b = f[i].getCentralDirectoryData();
+                        byte[] b = element.getCentralDirectoryData();
                         existing.parseFromCentralDirectoryData(b, 0, b.length);
                     }
                 }
@@ -593,7 +651,6 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
         }
     }
 
-    /** {@inheritDoc} */
     public Date getLastModifiedDate() {
         return new Date(getTime());
     }
@@ -601,6 +658,7 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
     /* (non-Javadoc)
      * @see java.lang.Object#equals(java.lang.Object)
      */
+    @Override
     public boolean equals(Object obj) {
         if (this == obj) {
             return true;
@@ -621,13 +679,13 @@ public class ZipArchiveEntry extends java.util.zip.ZipEntry
         String myComment = getComment();
         String otherComment = other.getComment();
         if (myComment == null) {
-            if (otherComment != null) {
-                return false;
-            }
-        } else if (!myComment.equals(otherComment)) {
-            return false;
+            myComment = "";
+        }
+        if (otherComment == null) {
+            otherComment = "";
         }
         return getTime() == other.getTime()
+            && myComment.equals(otherComment)
             && getInternalAttributes() == other.getInternalAttributes()
             && getPlatform() == other.getPlatform()
             && getExternalAttributes() == other.getExternalAttributes()
