@@ -21,25 +21,50 @@ package org.apache.commons.compress.compressors;
 import java.io.InputStream;
 
 public abstract class CompressorInputStream extends InputStream {
-    private int bytesRead = 0;
-    
+    private long bytesRead = 0;
+
+    /**
+     * Increments the counter of already read bytes.
+     * Doesn't increment if the EOF has been hit (read == -1)
+     * 
+     * @param read the number of bytes read
+     *
+     * @since 1.1
+     */
+    protected void count(int read) {
+        count((long) read);
+    }
+
     /**
      * Increments the counter of already read bytes.
      * Doesn't increment if the EOF has been hit (read == -1)
      * 
      * @param read the number of bytes read
      */
-    protected void count(int read) {
+    protected void count(long read) {
         if(read != -1) {
             bytesRead = bytesRead + read;
         }
     }
-    
+
     /**
      * Returns the current number of bytes read from this stream.
      * @return the number of read bytes
+     * @deprecated this method may yield wrong results for large
+     * archives, use #getBytesRead instead
      */
+    @Deprecated
     public int getCount() {
+        return (int) bytesRead;
+    }
+
+    /**
+     * Returns the current number of bytes read from this stream.
+     * @return the number of read bytes
+     *
+     * @since 1.1
+     */
+    public long getBytesRead() {
         return bytesRead;
     }
 }
