@@ -26,10 +26,13 @@ import org.apache.commons.compress.compressors.bzip2.BZip2CompressorInputStream;
 import org.apache.commons.compress.compressors.bzip2.BZip2CompressorOutputStream;
 import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;
 import org.apache.commons.compress.compressors.gzip.GzipCompressorOutputStream;
+import org.apache.commons.compress.compressors.lzma.LZMACompressorInputStream;
 import org.apache.commons.compress.compressors.xz.XZCompressorInputStream;
 import org.apache.commons.compress.compressors.xz.XZCompressorOutputStream;
+import org.apache.commons.compress.compressors.xz.XZUtils;
 import org.apache.commons.compress.compressors.pack200.Pack200CompressorInputStream;
 import org.apache.commons.compress.compressors.pack200.Pack200CompressorOutputStream;
+import org.apache.commons.compress.utils.IOUtils;
 
 /**
  * <p>Factory to create Compressor[In|Out]putStreams from names. To add other
@@ -46,7 +49,7 @@ import org.apache.commons.compress.compressors.pack200.Pack200CompressorOutputSt
  * cos.close();
  * </pre>
  * 
- * Example (Compressing a file):
+ * Example (Decompressing a file):
  * <pre>
  * final InputStream is = new FileInputStream(input); 
  * CompressorInputStream in = 
@@ -61,27 +64,52 @@ public class CompressorStreamFactory {
 
     /**
      * Constant used to identify the BZIP2 compression algorithm.
-     * @since Commons Compress 1.1
+     * @since 1.1
      */
     public static final String BZIP2 = "bzip2";
 
     /**
      * Constant used to identify the GZIP compression algorithm.
-     * @since Commons Compress 1.1
+     * @since 1.1
      */
     public static final String GZIP = "gz";
     /**
      * Constant used to identify the PACK200 compression algorithm.
-     * @since Commons Compress 1.3
+     * @since 1.3
      */
     public static final String PACK200 = "pack200";
 
     /**
      * Constant used to identify the XZ compression method.
-     * @since Commons Compress 1.4
+     * @since 1.4
      */
     public static final String XZ = "xz";
 
+    /**
+     * Constant used to identify the LZMA compression method.
+     * @since 1.6
+     */
+    public static final String LZMA = "lzma";
+
+    private boolean decompressConcatenated = false;
+
+    /**
+     * Whether to decompress the full input or only the first stream
+     * in formats supporting multiple concatenated input streams.
+     *
+     * <p>This setting applies to the gzip, bzip2 and xz formats only.</p>
+     *
+     * @param       decompressConcatenated
+     *                          if true, decompress until the end of the
+     *                          input; if false, stop after the first
+     *                          stream and leave the input position to point
+     *                          to the next byte after the stream
+     * @since 1.5
+     */
+    public void setDecompressConcatenated(boolean decompressConcatenated) {
+        this.decompressConcatenated = decompressConcatenated;
+    }
+
     /**
      * Create an compressor input stream from an input stream, autodetecting
      * the compressor type from the first few bytes of the stream. The InputStream
@@ -91,7 +119,7 @@ public class CompressorStreamFactory {
      * @return the compressor input stream
      * @throws CompressorException if the compressor name is not known
      * @throws IllegalArgumentException if the stream is null or does not support mark
-     * @since Commons Compress 1.1
+     * @since 1.1
      */
     public CompressorInputStream createCompressorInputStream(final InputStream in)
             throws CompressorException {
@@ -106,19 +134,20 @@ public class CompressorStreamFactory {
         final byte[] signature = new byte[12];
         in.mark(signature.length);
         try {
-            int signatureLength = in.read(signature);
+            int signatureLength = IOUtils.readFully(in, signature);
             in.reset();
 
             if (BZip2CompressorInputStream.matches(signature, signatureLength)) {
-                return new BZip2CompressorInputStream(in);
+                return new BZip2CompressorInputStream(in, decompressConcatenated);
             }
 
             if (GzipCompressorInputStream.matches(signature, signatureLength)) {
-                return new GzipCompressorInputStream(in);
+                return new GzipCompressorInputStream(in, decompressConcatenated);
             }
 
-            if (XZCompressorInputStream.matches(signature, signatureLength)) {
-                return new XZCompressorInputStream(in);
+            if (XZUtils.isXZCompressionAvailable() &&
+                XZCompressorInputStream.matches(signature, signatureLength)) {
+                return new XZCompressorInputStream(in, decompressConcatenated);
             }
 
             if (Pack200CompressorInputStream.matches(signature, signatureLength)) {
@@ -135,7 +164,7 @@ public class CompressorStreamFactory {
     /**
      * Create a compressor input stream from a compressor name and an input stream.
      * 
-     * @param name of the compressor, i.e. "gz", "bzip2", "xz", or "pack200"
+     * @param name of the compressor, i.e. "gz", "bzip2", "xz", "lzma", or "pack200"
      * @param in the input stream
      * @return compressor input stream
      * @throws CompressorException if the compressor name is not known
@@ -162,6 +191,10 @@ public class CompressorStreamFactory {
                 return new XZCompressorInputStream(in);
             }
 
+            if (LZMA.equalsIgnoreCase(name)) {
+                return new LZMACompressorInputStream(in);
+            }
+
             if (PACK200.equalsIgnoreCase(name)) {
                 return new Pack200CompressorInputStream(in);
             }
