@@ -69,13 +69,16 @@ public class GzipCompressorInputStream extends CompressorInputStream {
     private Inflater inf = new Inflater(true);
 
     // CRC32 from uncompressed data
-    private CRC32 crc = new CRC32();
+    private final CRC32 crc = new CRC32();
 
     private int memberSize;
 
     // True once everything has been decompressed
     private boolean endReached = false;
 
+    // used in no-arg read method
+    private final byte[] oneByte = new byte[1];
+
     /**
      * Constructs a new input stream that decompresses gzip-compressed data
      * from the specified input stream.
@@ -207,14 +210,13 @@ public class GzipCompressorInputStream extends CompressorInputStream {
     }
 
     private void readToNull(DataInputStream inData) throws IOException {
-        while (inData.readUnsignedByte() != 0x00) {}
+        while (inData.readUnsignedByte() != 0x00) { // NOPMD
+        }
     }
 
-    /** {@inheritDoc} */
     @Override
     public int read() throws IOException {
-        byte[] buf = new byte[1];
-        return read(buf, 0, 1) == -1 ? -1 : (buf[0] & 0xFF);
+        return read(oneByte, 0, 1) == -1 ? -1 : (oneByte[0] & 0xFF);
     }
 
     /**
