@@ -18,14 +18,17 @@
 
 package org.apache.commons.compress.archivers.zip;
 
+import static org.apache.commons.compress.AbstractTestCase.getFile;
+
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.net.URI;
-import java.net.URL;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Enumeration;
+import java.util.Iterator;
+import java.util.TreeMap;
 import java.util.zip.ZipEntry;
 
 import junit.framework.TestCase;
@@ -145,13 +148,94 @@ public class ZipFileTest extends TestCase {
      * @see "https://issues.apache.org/jira/browse/COMPRESS-176"
      */
     public void testWinzipBackSlashWorkaround() throws Exception {
-        URL zip = getClass().getResource("/test-winzip.zip");
-        File archive = new File(new URI(zip.toString()));
+        File archive = getFile("test-winzip.zip");
         zf = new ZipFile(archive);
         assertNull(zf.getEntry("\u00e4\\\u00fc.txt"));
         assertNotNull(zf.getEntry("\u00e4/\u00fc.txt"));
     }
 
+    /**
+     * Test case for 
+     * <a href="https://issues.apache.org/jira/browse/COMPRESS-208"
+     * >COMPRESS-208</a>.
+     */
+    public void testSkipsPK00Prefix() throws Exception {
+        File archive = getFile("COMPRESS-208.zip");
+        zf = new ZipFile(archive);
+        assertNotNull(zf.getEntry("test1.xml"));
+        assertNotNull(zf.getEntry("test2.xml"));
+    }
+
+    public void testUnixSymlinkSampleFile() throws Exception {
+        final String entryPrefix = "COMPRESS-214_unix_symlinks/";
+        final TreeMap<String, String> expectedVals = new TreeMap<String, String>();
+
+        // I threw in some Japanese characters to keep things interesting.
+        expectedVals.put(entryPrefix + "link1", "../COMPRESS-214_unix_symlinks/./a/b/c/../../../\uF999");
+        expectedVals.put(entryPrefix + "link2", "../COMPRESS-214_unix_symlinks/./a/b/c/../../../g");
+        expectedVals.put(entryPrefix + "link3", "../COMPRESS-214_unix_symlinks/././a/b/c/../../../\u76F4\u6A39");
+        expectedVals.put(entryPrefix + "link4", "\u82B1\u5B50/\u745B\u5B50");
+        expectedVals.put(entryPrefix + "\uF999", "./\u82B1\u5B50/\u745B\u5B50/\u5897\u8C37/\uF999");
+        expectedVals.put(entryPrefix + "g", "./a/b/c/d/e/f/g");
+        expectedVals.put(entryPrefix + "\u76F4\u6A39", "./g");
+
+        // Notice how a directory link might contain a trailing slash, or it might not.
+        // Also note:  symlinks are always stored as files, even if they link to directories.
+        expectedVals.put(entryPrefix + "link5", "../COMPRESS-214_unix_symlinks/././a/b");
+        expectedVals.put(entryPrefix + "link6", "../COMPRESS-214_unix_symlinks/././a/b/");
+
+        // I looked into creating a test with hard links, but zip does not appear to
+        // support hard links, so nevermind.
+
+        File archive = getFile("COMPRESS-214_unix_symlinks.zip");
+
+        zf = new ZipFile(archive);
+        Enumeration<ZipArchiveEntry> en = zf.getEntries();
+        while (en.hasMoreElements()) {
+            ZipArchiveEntry zae = en.nextElement();
+            String link = zf.getUnixSymlink(zae);
+            if (zae.isUnixSymlink()) {
+                String name = zae.getName();
+                String expected = expectedVals.get(name);
+                assertEquals(expected, link);
+            } else {
+                // Should be null if it's not a symlink!
+                assertNull(link);
+            }
+        }
+    }
+
+    /**
+     * @see "https://issues.apache.org/jira/browse/COMPRESS-227"
+     */
+    public void testDuplicateEntry() throws Exception {
+        File archive = getFile("COMPRESS-227.zip");
+        zf = new ZipFile(archive);
+
+        ZipArchiveEntry ze = zf.getEntry("test1.txt");
+        assertNotNull(ze);
+        assertNotNull(zf.getInputStream(ze));
+
+        int numberOfEntries = 0;
+        for (ZipArchiveEntry entry : zf.getEntries("test1.txt")) {
+            numberOfEntries++;
+            assertNotNull(zf.getInputStream(entry));
+        }
+        assertEquals(2, numberOfEntries);
+    }
+
+    /**
+     * @see "https://issues.apache.org/jira/browse/COMPRESS-228"
+     */
+    public void testExcessDataInZip64ExtraField() throws Exception {
+        File archive = getFile("COMPRESS-228.zip");
+        zf = new ZipFile(archive);
+        // actually, if we get here, the test already has passed
+
+        ZipArchiveEntry ze = zf.getEntry("src/main/java/org/apache/commons/compress/archivers/zip/ZipFile.java");
+        assertEquals(26101, ze.getSize());
+    }
+
     /*
      * ordertest.zip has been handcrafted.
      *
@@ -163,8 +247,7 @@ public class ZipFileTest extends TestCase {
      * central directory order is different from entry data order.
      */
     private void readOrderTest() throws Exception {
-        URL zip = getClass().getResource("/ordertest.zip");
-        File archive = new File(new URI(zip.toString()));
+        File archive = getFile("ordertest.zip");
         zf = new ZipFile(archive);
     }
 
